@@ -26,20 +26,41 @@ type KubeUser struct {
 // User contains the AuthInformation for a user
 type User struct {
 	// AuthProvider contains configuration for an external auth provider
-	AuthProvider AuthProvider `yaml:"auth-provider"`
+	AuthProvider *AuthProvider `yaml:"auth-provider,omitempty"`
+	// ExecProvider
+	ExecProvider *ExecProvider `yaml:"exec,omitempty"`
 }
+
 // AuthProvider cotnains the config and name of the kubeconfig auth provider plugin
 type AuthProvider struct {
 	// Cluster is the cluster identifier of the context
 	Config map[string]string `yaml:"config"`
 	// User is the user identifier of the context
-	Name    string `yaml:"name"`
+	Name string `yaml:"name"`
+}
+
+// ExecProvider
+type ExecProvider struct {
+	// APIVersion defines the versioned schema of this representation of an object.
+	// Servers should convert recognized schemas to the latest internal value, and
+	// may reject unrecognized values.
+	// More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources
+	// +optional
+	APIVersion string   `yaml:"apiVersion,omitempty" protobuf:"bytes,2,opt,name=apiVersion"`
+	Args       []string `yaml:"args"`
+	Command    string   `yaml:"command"`
+	Env        []EnvMap `yaml:"env"`
+}
+
+type EnvMap struct {
+	Name  string `yaml:"name"`
+	Value string `yaml:"value"`
 }
 
 // KubeCluster is a cluster configuration of a kubeconfig file
 type KubeCluster struct {
 	// Name is the name of the cluster
-	Name    string `yaml:"name"`
+	Name string `yaml:"name"`
 	// Cluster contains cluster configuration information
 	Cluster Cluster `yaml:"cluster"`
 }
@@ -48,9 +69,9 @@ type Cluster struct {
 	// CertificateAuthorityData contains CA info
 	CertificateAuthorityData string `yaml:"certificate-authority-data,omitempty"`
 	// Server is the API server address
-	Server                   string `yaml:"server"`
+	Server string `yaml:"server"`
 	// Insecure defines if the API server can be accessed with no CA checks
-	Insecure                 bool   `yaml:"insecure-skip-tls-verify,omitempty"`
+	Insecure bool `yaml:"insecure-skip-tls-verify,omitempty"`
 }
 
 // KubeConfig is a representation of a kubeconfig file
@@ -58,9 +79,9 @@ type Cluster struct {
 // used to show a preview of the Kubeconfig file
 type KubeConfig struct {
 	// TypeMeta common k8s type meta definition
-	TypeMeta       TypeMeta `yaml:",inline"`
+	TypeMeta TypeMeta `yaml:",inline"`
 	// CurrentContext is the current context of the kubeconfig file
-	CurrentContext string   `yaml:"current-context"`
+	CurrentContext string `yaml:"current-context"`
 	// Contexts are all defined contexts of the kubeconfig file
 	Contexts []KubeContext `yaml:"contexts"`
 	// Clusters are the cluster configurations
@@ -88,7 +109,7 @@ type TypeMeta struct {
 
 type KubeContext struct {
 	// Name is the name of the context
-	Name    string `yaml:"name"`
+	Name string `yaml:"name"`
 	// Context contains context configuration
 	Context Context `yaml:"context"`
 }
@@ -97,5 +118,5 @@ type Context struct {
 	// Cluster is the cluster identifier of the context
 	Cluster string `yaml:"cluster"`
 	// User is the user identifier of the context
-	User    string
+	User string
 }
